@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2015 Phusion Holding B.V.
+ *  Copyright (c) 2011-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -82,7 +82,7 @@ Controller::initializeFlags(Client *client, Request *req, RequestAnalysis &analy
 			part = part->next;
 		}
 
-		if (OXT_UNLIKELY(getLogLevel() >= LVL_DEBUG2)) {
+		if (OXT_UNLIKELY(LoggingKit::getLevel() >= LoggingKit::DEBUG2)) {
 			if (req->dechunkResponse) {
 				SKC_TRACE(client, 2, "Dechunk flag detected");
 			}
@@ -136,7 +136,7 @@ void
 Controller::initializePoolOptions(Client *client, Request *req, RequestAnalysis &analysis) {
 	boost::shared_ptr<Options> *options;
 
-	if (singleAppMode) {
+	if (req->config->singleAppMode) {
 		P_ASSERT_EQ(poolOptionsCache.size(), 1);
 		poolOptionsCache.lookupRandom(NULL, &options);
 		req->options = **options;
@@ -172,44 +172,38 @@ Controller::initializePoolOptions(Client *client, Request *req, RequestAnalysis
 				req->envvars->size);
 		}
 
+		// Allow certain options to be overridden on a per-request basis
 		fillPoolOption(req, req->options.maxRequests, PASSENGER_MAX_REQUESTS);
 	}
 }
 
 void
-Controller::fillPoolOptionsFromAgentsOptions(Options &options) {
-	options.ruby = defaultRuby;
-	if (agentsOptions->has("default_nodejs")) {
-		options.nodejs = agentsOptions->get("default_nodejs");
-	}
-	if (agentsOptions->has("default_python")) {
-		options.python = agentsOptions->get("default_python");
-	}
-	if (agentsOptions->has("meteor_app_settings")) {
-		options.meteorAppSettings = agentsOptions->get("meteor_app_settings");
-	}
+Controller::fillPoolOptionsFromConfigCaches(Options &options,
+	psg_pool_t *pool, const ControllerRequestConfigPtr &requestConfig)
+{
+	options.ruby = requestConfig->defaultRuby;
+	options.nodejs = requestConfig->defaultNodejs;
+	options.python = requestConfig->defaultPython;
+	options.meteorAppSettings = requestConfig->meteorAppSettings;
+	options.fileDescriptorUlimit = requestConfig->fileDescriptorUlimit;
 
-	options.logLevel = getLogLevel();
-	options.integrationMode = agentsOptions->get("integration_mode",
-		false, DEFAULT_INTEGRATION_MODE);
-	options.ustRouterAddress = ustRouterAddress;
+	options.logLevel = int(LoggingKit::getLevel());
+	options.integrationMode = psg_pstrdup(pool, mainConfig.integrationMode);
+	options.ustRouterAddress = requestConfig->ustRouterAddress;
 	options.ustRouterUsername = P_STATIC_STRING("logging");
-	options.ustRouterPassword = ustRouterPassword;
-	options.userSwitching = agentsOptions->getBool("user_switching");
-	if (agentsOptions->has("default_user")) {
-		options.defaultUser = agentsOptions->get("default_user");
-	}
-	if (agentsOptions->has("default_group")) {
-		options.defaultGroup = agentsOptions->get("default_group");
-	}
-	options.minProcesses = agentsOptions->getInt("min_instances");
-	options.maxPreloaderIdleTime = agentsOptions->getInt("max_preloader_idle_time");
-	options.maxRequestQueueSize = agentsOptions->getInt("max_request_queue_size");
-	options.abortWebsocketsOnProcessShutdown = agentsOptions->getBool("abort_websockets_on_process_shutdown");
-	options.forceMaxConcurrentRequestsPerProcess = agentsOptions->getInt("force_max_concurrent_requests_per_process");
-	options.spawnMethod = agentsOptions->get("spawn_method");
-	options.loadShellEnvvars = agentsOptions->getBool("load_shell_envvars");
-	options.statThrottleRate = statThrottleRate;
+	options.ustRouterPassword = requestConfig->ustRouterPassword;
+	options.userSwitching = mainConfig.userSwitching;
+	options.defaultUser = requestConfig->defaultUser;
+	options.defaultGroup = requestConfig->defaultGroup;
+	options.minProcesses = requestConfig->minInstances;
+	options.maxPreloaderIdleTime = requestConfig->maxPreloaderIdleTime;
+	options.maxRequestQueueSize = requestConfig->maxRequestQueueSize;
+	options.abortWebsocketsOnProcessShutdown = requestConfig->abortWebsocketsOnProcessShutdown;
+	options.forceMaxConcurrentRequestsPerProcess = requestConfig->forceMaxConcurrentRequestsPerProcess;
+	options.spawnMethod = requestConfig->spawnMethod;
+	options.loadShellEnvvars = requestConfig->loadShellEnvvars;
+	options.statThrottleRate = mainConfig.statThrottleRate;
+	options.maxRequests = requestConfig->maxRequests;
 
 	/******************************/
 }
@@ -341,7 +335,7 @@ Controller::createNewPoolOptions(Client *client, Request *req,
 		options.baseURI = StaticString(scriptName->start->data, scriptName->size);
 	}
 
-	fillPoolOptionsFromAgentsOptions(options);
+	fillPoolOptionsFromConfigCaches(options, req->pool, req->config);
 
 	const LString *appType = secureHeaders.lookup("!~PASSENGER_APP_TYPE");
 	if (appType == NULL || appType->size == 0) {
@@ -378,7 +372,9 @@ Controller::createNewPoolOptions(Client *client, Request *req,
 	fillPoolOption(req, options.restartDir, "!~PASSENGER_RESTART_DIR");
 	fillPoolOption(req, options.startupFile, "!~PASSENGER_STARTUP_FILE");
 	fillPoolOption(req, options.loadShellEnvvars, "!~PASSENGER_LOAD_SHELL_ENVVARS");
+	fillPoolOption(req, options.fileDescriptorUlimit, "!~PASSENGER_APP_FILE_DESCRIPTOR_ULIMIT");
 	fillPoolOption(req, options.raiseInternalError, "!~PASSENGER_RAISE_INTERNAL_ERROR");
+	fillPoolOption(req, options.lveMinUid, "!~PASSENGER_LVE_MIN_UID");
 	/******************/
 
 	boost::shared_ptr<Options> optionsCopy = boost::make_shared<Options>(options);
@@ -452,7 +448,7 @@ Controller::getStickySessionCookieName(Request *req) {
 	const LString *value = req->headers.lookup(PASSENGER_STICKY_SESSIONS_COOKIE_NAME);
 	if (value == NULL || value->size == 0) {
 		return psg_lstr_create(req->pool,
-			defaultStickySessionsCookieName);
+			req->config->defaultStickySessionsCookieName);
 	} else {
 		return value;
 	}
@@ -477,13 +473,15 @@ Controller::onRequestBegin(Client *client, Request *req) {
 		// and localize them as much as possible, for better CPU caching.
 		RequestAnalysis analysis;
 		analysis.flags = req->secureHeaders.lookup(FLAGS);
-		analysis.appGroupNameCell = singleAppMode
+		analysis.appGroupNameCell = req->config->singleAppMode
 			? NULL
 			: req->secureHeaders.lookupCell(PASSENGER_APP_GROUP_NAME);
 		analysis.unionStationSupport = unionStationContext != NULL
 			&& getBoolOption(req, UNION_STATION_SUPPORT, false);
 		req->stickySession = getBoolOption(req, PASSENGER_STICKY_SESSIONS,
-			this->stickySessions);
+			mainConfig.stickySessions);
+		req->showVersionInHeader = getBoolOption(req, PASSENGER_SHOW_VERSION_IN_HEADER,
+			req->config->showVersionInHeader);
 		req->host = req->headers.lookup(HTTP_HOST);
 
 		/***************/
